@@ -3,7 +3,7 @@
 """
 Access to RDF stores.
 
-Copyright (C) 2005, 2006 Paul Boddie <paul@boddie.org.uk>
+Copyright (C) 2005, 2006, 2007 Paul Boddie <paul@boddie.org.uk>
 
 This software is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -140,6 +140,38 @@ class Store:
             urirefs += handler.get_urirefs_for_attribute(attribute)
         return urirefs
 
+    def get_predicates_for_attribute(self, attribute):
+
+        """
+        Query the handlers and return a tuple of predicates which can represent
+        the given 'attribute'.
+        """
+
+        urirefs = self.get_urirefs_for_attribute(attribute)
+        if not urirefs:
+            return ()
+
+        # Convert the URIRefs into a list of predicate lists.
+
+        predicates = [[p] for p in urirefs[0]]
+
+        # For each URIRef list, traverse from right to left, combining the
+        # contents with the predicates, adding extra elements on the left if
+        # necessary.
+
+        for l in urirefs[1:]:
+            np = len(predicates)
+            for i in range(0, len(l)):
+                uriref = l[-1-i]
+                predicate = predicates[-1-i]
+                if i >= np:
+                    predicates.insert(0, uriref)
+                else:
+                    if uriref not in predicate:
+                        predicate.append(uriref)
+
+        return tuple(predicates)
+
     # Standard store methods.
 
     def get_context(self, context):
@@ -212,26 +244,6 @@ class Store:
                 expressions.append("_ = ?")
         return self.store.Expression("(" + " or ".join(expressions) + ")", values, object_type)
 
-    def get_owner_items_using_query(self, property_types, property_values, comparison):
-
-        # NOTE: Permits negation.
-
-        if len(property_values) == 0:
-            query = self.get_comparison(None, [self.store.Defined()])
-        else:
-            query = self.get_comparison(comparison, property_values)
-
-        # Build the hierarchy.
-
-        query = self.store.subjects(pattern=((None,) + tuple(property_types) + (query,)))
-
-        # NOTE: The actual negation.
-
-        if len(property_values) == 0:
-            query = self.store.not_subjects(query)
-
-        return query
-
     def get_item_type(self, item):
 
         "Return the type of the given 'item'."
@@ -529,14 +541,27 @@ class Store:
     def get_identifier_from_item_type(self, item_type):
         return self.get_label(item_type)
 
-    def get_property_types_from_identifier(self, identifier):
+    def get_urirefs_from_identifier(self, identifier):
+
+        """
+        Return a list of URIRef objects corresponding to the given property type
+        'identifier' or None if any URIRef objects should correspond (when
+        'identifier' is specified as None).
+        """
+
         if identifier == "item-type":
             return [self.TYPE]
         elif identifier is None:
-            return [None]
+            return None
         else:
             return self.get_urirefs_for_property_type(identifier)
 
+    def get_item_types_from_identifiers(self, values):
+        l = []
+        for value in values:
+            l += self.get_values_from_identifier(self.TYPE, value)
+        return l
+
     def get_values_from_identifier(self, property_type, value):
         if value is None:
             return [None]
@@ -549,6 +574,13 @@ class Store:
             else:
                 return [self.Literal(value)]
 
+    def get_values_using_predicates(self, predicates, value):
+        property_types = predicates[-1]
+        values = []
+        for property_type in property_types:
+            values += self.get_values_from_identifier(property_type, value)
+        return values
+
     # Querying methods.
 
     def get_selected_items(self, values):
@@ -563,40 +595,6 @@ class Store:
             items.append(self.get_item_from_identifier(value))
         return unique(items)
 
-    def get_filtered_items(self, filters):
-
-        """
-        Return a list of items filtered using information from 'filters', which
-        consists of a list of 2-tuples, each containing a list of property types
-        (eg. ["http://.../uri"], ["http://.../dtstart", "http://.../datetime"])
-        and either a list of simple values (eg. ["20050827", "20050828"]) or a
-        2-tuple describing the comparison and a single value (eg. ("startswith",
-        "200508")).
-        """
-
-        new_filters = []
-        for property_types, values in filters:
-
-            # Get suitably prepared values.
-
-            property_values = []
-            if isinstance(values, tuple):
-                comparison, value = values
-                property_values += self.get_values_from_identifier(property_types[-1], value)
-            else:
-                comparison = None
-                for value in values:
-                    property_values += self.get_values_from_identifier(property_types[-1], value)
-
-            new_filters.append((property_types, property_values, comparison))
-
-        # Produce queries for each of the filters.
-
-        queries = []
-        for property_types, property_values, comparison in new_filters:
-            queries.append(self.get_owner_items_using_query(property_types, property_values, comparison=comparison))
-        return self.store.or_subjects(queries)
-
     def combine_items(self, results):
 
         "Combine the 'results', returning a collection of items."
@@ -609,6 +607,12 @@ class Store:
         To query the store, prepare filters, taking information from the given
         'attributes' and adding the default item types where appropriate, and
         return a list of items.
+
+        Supported attributes:
+
+        filter-item-value, filter-type
+        search-value, search-type, search-operation
+        (attributes supported by get_date_from_attributes)
         """
 
         results = []
@@ -616,52 +620,64 @@ class Store:
         # Use filtering categories.
 
         if attributes.has_key("filter-item-value"):
-            filters = []
-            filter_item_value = attributes["filter-item-value"]
+
+            predicates = self.get_predicates_for_attribute(attributes["filter-type"])
+
+            filter_item_values = self.get_values_using_predicates(
+                predicates,
+                attributes["filter-item-value"]
+                )
 
             # Ask the handlers for criteria.
+            # Make a pattern requesting all suitable items.
 
-            for urirefs in self.get_urirefs_for_attribute(attributes["filter-type"]):
-                filters.append((urirefs, [filter_item_value]))
-            results.append(self.get_filtered_items(filters))
+            pattern = (None,) + \
+                predicates + \
+                (self.get_comparison(None, filter_item_values),)
+
+            results.append(self.store.subjects(pattern=pattern))
 
         elif attributes.has_key("search-value"):
-            filters = []
-            search_value = attributes["search-value"]
+
+            predicates = self.get_predicates_for_attribute(attributes["search-type"])
+
+            search_values = self.get_values_using_predicates(
+                predicates,
+                attributes["search-value"]
+                )
+
+            op = attributes.get("search-operation")
 
             # Ask the handlers for criteria.
+            # Make a pattern requesting all suitable items.
 
-            for urirefs in self.get_urirefs_for_attribute(attributes["search-type"]):
-                op = attributes.get("search-operation")
-                if op in (None, "exact"):
-                    filters.append((urirefs, [search_value]))
-                else:
-                    filters.append((urirefs, (op, search_value)))
-            results.append(self.get_filtered_items(filters))
+            pattern = (None,) + \
+                predicates + \
+                (self.get_comparison(op, search_values),)
 
-        # NOTE: Hack used to optimise sqltriples-based querying.
-
-        elif getattr(self.store, "supports_querying", 0):
-            filters = []
-
-            # Ask the handlers for criteria.
-
-            for urirefs in self.get_urirefs_for_attribute("related-to"):
-                filters.append((urirefs, []))
-            results.append(self.get_filtered_items(filters))
+            results.append(self.store.subjects(pattern=pattern))
 
         # Date-based filtering.
 
         date = self.get_date_from_attributes(**attributes)
         if date is not None:
-            pattern = ("startswith", date)
-            filters = []
+
+            # For date predicate restrictions try this:
+            # self.store.Expression(
+            #        "_ not in ?",
+            #        [self.get_urirefs_for_property_type("created") + self.get_urirefs_for_property_type("last-modified")]
+            #        )
 
             # Ask the handlers for datetime-related criteria.
 
-            for uriref in self.get_urirefs_for_property_type("datetime"):
-                filters.append(([None, uriref], pattern))
-            results.append(self.get_filtered_items(filters))
+            pattern = (
+                None,
+                None,
+                self.get_urirefs_for_property_type("datetime"),
+                self.get_comparison("startswith", [date])
+                )
+
+            results.append(self.store.subjects(pattern=pattern))
 
         # Or use type-based criteria.
 
@@ -670,13 +686,15 @@ class Store:
         else:
             item_types = self.default_item_types
 
-        filters = []
-
         # Ask the handlers for type-related criteria.
 
-        for uriref in self.get_property_types_from_identifier("item-type"):
-            filters.append(([uriref], item_types))
-        results.append(self.get_filtered_items(filters))
+        pattern = (
+            None,
+            self.get_urirefs_from_identifier("item-type"),
+            self.get_item_types_from_identifiers(item_types),
+            )
+
+        results.append(self.store.subjects(pattern=pattern))
 
         # Combine the results.
 
