@@ -3,7 +3,7 @@
 """
 Viewer classes.
 
-Copyright (C) 2005, 2006 Paul Boddie <paul@boddie.org.uk>
+Copyright (C) 2005, 2006, 2007 Paul Boddie <paul@boddie.org.uk>
 
 This software is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -33,8 +33,6 @@ class IndexViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
     "A portal index page viewer."
 
     resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
-    encoding = "utf-8"
-    path_encoding = "utf-8"
     template_resources = {
         "index" : ("index_template.xhtml", "index_output.xsl")
         }
@@ -109,7 +107,7 @@ class IndexViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         # Start the response.
 
-        trans.set_content_type(ContentType("application/xhtml+xml", self.encoding))
+        trans.set_content_type(ContentType("application/xhtml+xml"))
 
         # Define the stylesheet parameters.
 
